<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * XLite\Model\Repo\Product class tests
 *
 * @category   LiteCommerce
 * @package    Tests
 * @subpackage Classes
 * @author     Creative Development LLC <info@cdev.ru>
 * @copyright  Copyright (c) 2010 Creative Development LLC <info@cdev.ru>. All rights reserved
 * @license    http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 * @link       http://www.litecommerce.com/
 * @see        ____file_see____
 * @since      1.0.0
 */

include_once dirname(__DIR__) . '/AProduct.php';

class XLite_Tests_Model_Repo_Product extends XLite_Tests_Model_AProduct
{
   /**
     * Return data needed to start application.
     * Derived class can redefine this method.
     * It's possible to detect current test using the $this->name variable
     *
     * @return array
     * @access protected
     * @see    ____func_see____
     * @since  1.0.0
     */
    protected function getRequest()
    {
        $request = parent::getRequest();
        // Except some ones, all methods are emulate the customer area
        $request['controller'] = ('testSearchDisabledItems' === $this->getName());

        return $request;
    }

    /**
     * getDefaultCnd
     *
     * @return \XLite\Core\CommonCell
     * @access protected
     * @see    ____func_see____
     * @since  1.0.0
     */
    protected function getDefaultCnd()
    {
        return new \XLite\Core\CommonCell(
            array(
                \XLite\Model\Repo\Product::P_SKU => '',
                \XLite\Model\Repo\Product::P_CATEGORY_ID => 1,
                \XLite\Model\Repo\Product::P_SUBSTRING => '',
                \XLite\Model\Repo\Product::P_SEARCH_IN_SUBCATS => true,
                \XLite\Model\Repo\Product::P_ORDER_BY => array(
                    \XLite\View\ItemsList\Product\AProduct::SORT_BY_MODE_NAME,
                    \XLite\View\ItemsList\Product\AProduct::SORT_ORDER_ASC
                ),
                \XLite\Model\Repo\Product::P_LIMIT => array(0, 100),
            )
        );
    }

    /**
     * Get array of all fields allowable for searching
     * 
     * @return array
     * @see    ____func_see____
     * @since  1.0.0
     */
    protected function getSearchFields()
    {
        return array(
            \XLite\Model\Repo\Product::P_SKU,
            \XLite\Model\Repo\Product::P_CATEGORY_ID,
            \XLite\Model\Repo\Product::P_SUBSTRING,
            \XLite\Model\Repo\Product::P_SEARCH_IN_SUBCATS,
            \XLite\Model\Repo\Product::P_INVENTORY,
            \XLite\Model\Repo\Product::P_INCLUDING,
            \XLite\Model\Repo\Product::P_LIMIT,
            // The field below is disabled because of error generated by Doctrine:
            // Doctrine\ORM\Query\QueryException: [Syntax Error] line 0, col 301: Error: Expected end of string, got '<'
            // \XLite\Model\Repo\Product::P_ORDER_BY, 
        );
    }


	/**
	 * testSearchAll
	 *
	 * @return void
	 * @access public
	 * @see    ____func_see____
	 * @since  1.0.0
	 */
	public function testSearchAll()
	{
        $cnd = $this->getDefaultCnd();

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd);

        // If all products selected
        $this->assertEquals(8, count($result), 'Number of found product does not match');
        // If the first selected (SORT_BY_MODE_NAME, SORT_ORDER_ASC) is the "Apple" one
        $this->assertEquals(15090, $result[0]->getProductId(), 'ID of the first found product does not match');
	}

    /**
     * testSearchBySubstring
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testSearchBySubstring()
    {
        $cnd = $this->getDefaultCnd();
        $cnd->{\XLite\Model\Repo\Product::P_SUBSTRING} = 'botanical';

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd);

        // 2 products should be found
        $this->assertEquals(2, count($result), 'Number of found product does not match');
        // Check if the "Pea" and "Peach" products are only selected.
        // Also, the order is checked (SORT_BY_MODE_NAME, SORT_ORDER_ASC)
        $this->assertEquals(16280, $result[0]->getProductId(), 'ID of the first found product does not match'); // Pea
        $this->assertEquals(15091, $result[1]->getProductId(), 'ID of the first second product does not match'); // Peach


        $cnd->{\XLite\Model\Repo\Product::P_SUBSTRING} = 'Apple Radish';

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(0, $result, 'There should be no Apple Radish by default');


        $cnd->{\XLite\Model\Repo\Product::P_INCLUDING} = \XLite\Model\Repo\Product::INCLUDING_ANY;

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(3, $result, 'There should be 3 "Apple or Radish" products');


        $cnd->{\XLite\Model\Repo\Product::P_INCLUDING} = \XLite\Model\Repo\Product::INCLUDING_ALL;

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(0, $result, 'There should be no Apple AND Radish products');


        $cnd->{\XLite\Model\Repo\Product::P_SUBSTRING} = 'Radish sativus';

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(1, $result, 'There should be 1 "Radish AND sativus" product');


        $cnd->{\XLite\Model\Repo\Product::P_SUBSTRING} = 'Radish "consumed throughout"';

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(1, $result, 'There should be 1 "Radish AND sativus" product');

        // E:0039975
        $cnd->{\XLite\Model\Repo\Product::P_SUBSTRING} = 'Apple Radish';
        $cnd->{\XLite\Model\Repo\Product::P_INCLUDING} = '1<ScRiPt%20%0d%0a>alert(/xss15212/.source)%3B</ScRiPt>';
        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        $this->assertEquals(0, $result, 'There should be 0 products on substring "Apple Radish" and non-standard "includes" value');

        // Check for error exceptions
        $fields = $this->getSearchFields();

        foreach ($fields as $field) {

            $cnd = $this->getDefaultCnd();

            if (in_array($field, array(\XLite\Model\Repo\Product::P_LIMIT, \XLite\Model\Repo\Product::P_ORDER_BY))) {
                $cnd->{$field} = array(
                    '1<ScRiPt%20%0d%0a>alert(/xss15212/.source)%3B</ScRiPt>',
                    '1<ScRiPt%20%0d%0a>alert(/xss15212/.source)%3B</ScRiPt>',
                );

            } else {
                $cnd->{$field} = '1<ScRiPt%20%0d%0a>alert(/xss15212/.source)%3B</ScRiPt>';
            }

            $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd);

            $this->assertInternalType('array', $result, sprintf('Check that result is an array failed (%s)', $field));
        }
    }

    /**
     * testSearchInSingleCategory
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testSearchInSingleCategory()
    {
        $cnd = $this->getDefaultCnd();
        $cnd->{\XLite\Model\Repo\Product::P_CATEGORY_ID} = 14009;
        $cnd->{\XLite\Model\Repo\Product::P_SEARCH_IN_SUBCATS} = false;

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd);

        // 4 products should be found
        $this->assertEquals(3, count($result), 'Number of found product does not match');

        // Check category IDs
        foreach ($result as $entity) {
            $this->assertEquals(14009, $entity->getCategoryId(), 'Category ID of the product "' . $entity->getName() . '" does not match');
        }
    }

    /**
     * testSearchItemsCount
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testSearchItemsCount()
    {
        $cnd = $this->getDefaultCnd();
        $cnd->{\XLite\Model\Repo\Product::P_CATEGORY_ID} = 14015;

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        // 5 products should be found
        $this->assertEquals(5, $result, 'Number of found product does not match');
    }

    /**
     * testSearchDisabledItems
     * NOTE: this method is emulate the admin area
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testSearchDisabledItems()
    {
        $cnd = $this->getDefaultCnd();

        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->search($cnd, true);

        // 9 products should be found
        $this->assertEquals(9, $result, 'Number of found product does not match');
    }

    /**
     * testFindByCleanUrlExists
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testFindByCleanUrlExists()
    {
        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->findOneByCleanURL('test');

        // Certain product is found
        $this->assertNotNull($result, 'Product not found');
        $this->assertEquals(16282, $result->getProductId(), 'Product ID does not match');
    }

    /**
     * testFindByCleanUrlNotExists
     *
     * @return void
     * @access public
     * @see    ____func_see____
     * @since  1.0.0
     */
    public function testFindByCleanUrlNotExists()
    {
        $result = \XLite\Core\Database::getRepo('\XLite\Model\Product')->findOneByCleanURL('not_exists');

        // Product was not found
        $this->assertNull($result, 'Wrong result for the search');
    }
}
